// Array Multiplier. Licensed under the terms of the GNU GPL version 2.0 or
// higher.
module full_adder (x, y, cin, cout, s);
    input x, y, cin;
    output s, cout;
    assign cout = x&y | x&cin | y&cin;
    assign s = y ^ cin ^ x;
endmodule

module half_adder(a,b, cout,s)
  input a,b;
  output s,cout;
  assign s = a^b;
  assign cout = a&b;
endmodule

module double_adder(a,b,c,d, cin,cout, x,y);
  input a,b,c,d,cin;
  output x,y,cout;
  wire p,q;
  full_adder faone(a,b,c, cout,p);
  full_adder fatwo(p,d,cin, x,y);
endmodule

module array_cell (ppiin, mj, mjn, plusi, minusi, cin, cout, ones, ppip1out);
    input ppiin, mj, mjn, plusi, minusi, cin;
    output cout, ppip1out, ones;
    assign ones = (plusi & mj) | (minusi & mjn);
    full_adder fa (ppiin, ones, cin, cout, ppip1out);
endmodule

module booth_decoder (pos,neg,in,out);
  input pos,neg,in;
  output out;
  assign out = (pos&in)|(neg&~in);
endmodule

module booth_encoder (qim1, qi, plusi, minusi);
    input qim1, qi;
    output  plusi, minusi;
    assign plusi = qim1 & ~qi;
    assign minusi = ~qim1 & qi;
endmodule

module decode_row(M,pctrl,mctrl,DEC)
  parameter n=8;
  input [n-1:0]M;
  input pctrl,mctrl;
  output [n:0]DEC;

  generate
    genvar i;
    for (i=0;i<n;i=i+1)
    begin:row
      booth_decoder bd(M[i],pctrl,mctrl,DEC[0]);
    end
  endgenerate
  
  assign DEC[n] = (DEC[7]&DEC[6]&DEC[5]&DEC[4]&DEC[3]&DEC[2]&DEC[1]&DEC[0]&mctrl)^DEC[7];
endmodule

module mult_row (PPI,PPO,pctrl,mctrl,M);
  parameter n=8;
  input [n:0] PPI;
  output [n+1:0] PPO;
  input pctrl,mctrl;
  input [n-1:0] M;
  wire [n-1:0] O;
  wire [n-1:-1] no;
  wire [n:0] carry;
  wire twos;

  generate
    genvar i;
    for (i=0; i<n; i=i+1)
    begin:col
      array_cell ac(PPI[i], M[i], ~M[i], pctrl, mctrl, carry[i], carry[i+1], O[i], PPO[i]);
      assign no[i] = no[i-1] & O[i];
    end
  endgenerate

  assign carry[0] = mctrl;
  //sign extension, is the ones sign (flip only if all ones and we are negating)
  assign twos = O[n-1] ^ (mctrl & no[n-2]);
  full_adder fa(~twos, carry[n], PPI[n], PPO[n+1], PPO[n]);
endmodule

module bonus(M,Q,P);
  parameter n=8;
  input [n-1:0] M,Q;
  output [2*n-1:0] P;
  wire [n:0] s0,s1,s2,s3,s4,s5,s6,s7,s8,s9;
  wire [6:0] t0,t1,t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12,t13,t14,t15,t16;
  wire [7:0] u0,u1,u2,u3,u4,u5,u6,u7,u8,u9,u10,u11,u12,u13,u14,u15,u16;
  wire [n-1:0] pctrl, mctrl;
  wire [n:0] beIn;
  wire [n+1:0] s0,sf;
  assign beIn[n:1] = Q[n-1:0];
  assign beIn[0] = 0;

  generate
    genvar i;
    for (i=0; i<n; i=i+1)
    begin:bes
      booth_encoder be(beIn[i],beIn[i+1],pctrl[i],mctrl[i]);
    end
  endgenerate

  decode_row dr1(M,pctrl[0],mctrl[0],s0);
  decode_row dr2(M,pctrl[1],mctrl[1],s1);
  decode_row dr3(M,pctrl[2],mctrl[2],s2);
  decode_row dr4(M,pctrl[3],mctrl[3],s3);
  decode_row dr5(M,pctrl[4],mctrl[4],s4);
  decode_row dr6(M,pctrl[5],mctrl[5],s5);
  decode_row dr7(M,pctrl[6],mctrl[6],s6);
  decode_row dr8(M,pctrl[7],mctrl[7],s7);

  half_adder   a1(s0[0],mctrl[0], t1[0],t0[0]);
  full_adder   a2(s0[1],s1[0],mctrl[1], t2[5], t1[1]);
  double_adder a3(s0[2],s1[1],s2[0],mctrl[2], t2[5],t3[5], t3[0], t2[0]);
  double_adder a4(s0[3],s1[2],s2[1],s3[0], t3[5],t4[5], t4[0],t3[1]);
  double_adder a5(s0[4],s1[3],s2[2],s3[1], t4[5],t5[5], t5[0],t4[1]);
  double_adder a6(s0[5],s1[4],s2[3],s3[2], t5[5],t6[5], t6[0],t5[1]);
  double_adder a7(s0[6],s1[5],s2[4],s3[3], t6[5],t7[5], t7[0],t6[1]);
  double_adder a8(s0[7],s1[6],s2[5],s3[4], t7[5],t8[5], t8[0],t7[1]);
  double_adder a9(~s0[8],s1[7],s2[6],s3[5], t8[5],t9[5], t9[0],t8[1]);
  double_adder a10(s0[8],s1[8],s2[7],s3[6], t9[5],t10[5], t10[0],t9[1]);
  double_adder a11(s2[8],s3[7],s4[6],s5[5], t10[5],t11[5], t11[0],t10[1]);
  double_adder a12(s3[8],s4[7],s5[6],s6[5], t11[5],t12[5], t12[0],t11[1]);
  double_adder a13(s4[8],s5[7],s6[6],s7[5], t12[5],t13[1], t13[0],t12[1]);
  full_adder   a14(s5[8],s6[7],s7[6], t14[5],t13[2]);
  full_adder   a15(s6[8],s7[7], t14[5],t15[0],t14[0]);
  full_adder   b6(s4[1],s5[0],mctrl[5], t6[6],t5[2]);
  double_adder b7(s4[2],s5[1],s6[0],mctrl[6], t6[6],t7[6], t7[2],t6[2]);
  double_adder b8(s4[3],s5[2],s6[1],s7[0], t7[6],t8[6], t8[2],t7[3]);
  double_adder b9(s4[4],s5[3],s6[2],s7[1], t8[6],t9[6], t9[2],t8[3]);
  double_adder b10(s4[5],s5[4],s6[3],s7[2], t9[6],t10[6], t10[2],t9[3]);
  full_adder   b11(s6[4],s7[3],t10[6], t11[2], t10[3]);

  assign P[0] = t0[0];

  half_adder   c2(t1[0],t1[1], u2[5], P[1]);
  half_adder   c3(t2[0],u2[5], u3[5], P[2]);
  double_adder c4(mctrl[3],t3[0],t3[1],0,u3[5], u4[5], u4[0], P[3]);
  double_adder c5(mctrl[4],s4[0],t4[1],t4[0],u4[5], u5[5], u5[0], u4[1]);
  double_adder c6(0,t5[2],t5[1],t5[0],u5[5], u6[5], u6[0], u5[1]);
  double_adder c7(0,t6[2],t6[1],t6[0],u6[5], u7[1], u7[0], u6[1]);
  double_adder c8(t7[3],t7[2],t7[1],t7[0],mctrl[7], u8[5], u8[0], u7[2]);
  double_adder c9(t8[3],t8[2],t8[1],t8[0],u8[5], u9[5], u9[0], u8[1]);
  double_adder c10(t9[3],t9[2],t9[1],t9[0],u9[5], u10[5], u10[0], u9[1]);
  double_adder c11(t10[3],t10[2],t10[1],t10[0],u10[5], u11[5], u11[0], u10[1]);
  double_adder c12(s7[4],t11[2],t11[1],t11[0],u11[5], u12[5], u12[0], u11[1]);
  full_adder   c13(t12[1],t12[0],u12[5], u13[5], u12[1]);
  full_adder   c14(t13[1],t13[0],u13[5], u14[5], u13[0]);
  half_adder   c15(t14[0],u14[5], u15[5], u14[0]);
  full_adder   c16(t15[0],u15[5],s7[8], u15[6], u15[0]);

  half_adder   d5(u4[0],u4[1], u5[6], P[4]);
  full_adder   d6(u5[0],u5[1],u5[6], u6[6], P[5]);
  full_adder   d7(u6[0],u6[1],u6[6], u7[6], P[6]);
  double_adder d8(0,u7[0],u7[1],u7[2],u7[6], u8[6],u8[7], P[7]);
  double_adder d9(0,u8[0],u8[1],u8[7],u8[6], u8[6],u8[7], P[8]);
  double_adder d10(0,u9[0],u9[1],u9[7],u9[6], u10[6],u10[7], P[9]);
  double_adder d11(0,u10[0],u10[1],u10[7],u10[6], u11[6],u11[7], P[10]);
  double_adder d12(0,u11[0],u11[1],u11[7],u11[6], u12[6],u12[7], P[11]);
  double_adder d13(0,u12[0],u12[1],u12[7],u12[6], u13[6],u13[7], P[12]);
  full_adder   d14(u13[0],u13[6],u13[7], u14[6], P[13]);
  full_adder   d15(u14[0],u14[6],u14[7], u15[7], P[14]);
  half_adder   d16(u15[0],u15[7], u15[4], P[15]);

endmodule
