#include <stdio.h>
#include "poke.h"

/*
 CFG_PORT:  controls the START and STOP flip-flops
 TIME_PORT: read the reaction time from this port
 */

#define CFG_PORT   0xB00000
#define TIME_PORT  0xB00000

#define ICR        0xFFFFFFFA
#define CFPGA_ICR  0x00C20000
#define PDATA      0xFFFFFFF0
#define AUTO6      0x00000078

#define USE_PRINTF

extern int intrstub( void );
extern int spl0( void ), spl7( void );

void int2string(int number, char *string);

int main( )
{ 
	int i;
		 
	/* 
     Setup the interrupts on the Ultragizmo board 
   */
   spl7( ); 
   pokeb( CFPGA_ICR, 0x06 );
   pokel( AUTO6    , intrstub );
   pokew( ICR      , 0x60ff );
   pokew( PDATA    , 0xfff7 );
   spl0( );

	puts("Welcome to the reaction timer.");
	puts("Flick the switch when you see the lightsturn on.\nGood luck!\n");

	/* Turn LED on after a period of time */
  	for (i = 0; i < 3500000; i++);

	/* Start timer */
	pokew(TIME_PORT, 1);

   for(;;); /* Wait forever */
}

int interrupt( )
{
	char time_str[100];
	unsigned short time;
	time = peekw( TIME_PORT );
 	 
	/* Clear interupt */	
	pokew(TIME_PORT, 0);	

	/* Display the time out. */
#ifdef USE_PRINTF
	printf("Time was: %dms\n\n", time);
#else
	int2string(time, time_str);
	puts("Time was: ");
	puts(time_str);
	puts("ms\n\n");
#endif

	exit(0);
}

/* Conversts an int to a string. */
void int2string(int number, char *string)
{
	int i, divby;	
	char temp;
	int done_zeros = 0;

	/* FPGA Counter can only store 65 seconds. */
	number %= 65000;
	divby = 10000;

	/* Main look for extracting each digit. */	
	for (i = 0; i < 5; i++)
	{
		/* Ignore leading zeros. */
	 	if (done_zeros)
			*string++ = (char) (0x30 + (number / divby));
	  	else
	 	{
	  		temp = (char) (0x30 + (number / divby));
			/* Check if this digit is still a leading zero. */
	 		if (temp != '0')
	 		{
	 			*string++ = temp;
	 			done_zeros = 1;
	 		}
	 	}
		number %= divby;
		divby /= 10;
	}
	
	/* Add NULL character to end, to mark end of string. */
	*string = '\0';
}
