/*Top-Level Wrapper File to implement circuits with the FLEX10K70
on the Ultragizmo board. DO NOT change any of the pin definitions
unless you are completely sure of the consequences.
*/
module wrapper (address, data, codata, srama, sramdl, sramdh, codsync,
                codread, codwrite, sram1en, sram2en, sram1oe, sram2oe,
                sram1we, sram2we, sram1ud, sram2ud, sram1ld, sram2ld,
                br_sfpga, bg_sfpga, bgack_sfpga, clk, fc, berr, as, halt, reset,
                dtack, lds, uds, rw, irqsf, iacksf, hex0, hex1, hex2, hex3,
                sfpga_digital, sfpga_con40, sfpga_logic, sfpga_con60, led, pclk
              );

input [23:1] address;
inout [15:0] data;
input [31:0] codata;
input [17:0] srama;
input [15:0] sramdl;
input [15:0] sramdh;
input codsync, codread, codwrite, sram1en, sram2en, sram1oe, sram2oe;
input sram1we, sram2we, sram1ud, sram2ud, sram1ld, sram2ld;
input br_sfpga, bg_sfpga, bgack_sfpga, clk;
input [2:0] fc;
input berr, as, halt, reset;
inout dtack;
input lds, uds, rw, iacksf;
input [33:0] sfpga_digital;
input [31:0] sfpga_con40;
input [17:0] sfpga_logic;
input [51:0] sfpga_con60;
input pclk; 

output [7:0] hex0, hex1, hex2, hex3;
output [15:0] led;
output irqsf;

  // Put Code Here
  wire w_reg,r_reg,c_en,d_clk;
  wire [15:0]Q;
  reg start,stop;
  
  bus_iface bi( address, as, uds, lds, rw, dtack, w_reg, r_reg );
  clk_div   cd( pclk, d_clk );
  cntr      ct( d_clk, c_en, stop, Q );

  assign data = (r_reg) ? Q : 16'bZ;

  always @(posedge w_reg)
  begin
    start <= data[0];
    stop  <= data[1];
  end
  always @(posedge sfpga_digital[0])
    stop <= 1;
  
  assign irqsf = ~stop | start;
  assign c_en  = ~stop & start;
  assign led[0] = ~stop & start;
  assign led[1] = ~stop & start;
  assign led[2] = ~stop & start;
  
endmodule




