// Array Multiplier. Licensed under the terms of the GNU GPL version 2.0 or
// higher.
module full_adder (x, y, cin, cout, s);
    input x, y, cin;
    output s, cout;
    assign cout = x&y | x&cin | y&cin;
    assign s = y ^ cin ^ x;
endmodule

module array_cell (ppiin, mj, mjn, plusi, minusi, cin, cout, ones, ppip1out);
    input ppiin, mj, mjn, plusi, minusi, cin;
    output cout, ppip1out, ones;
    assign ones = (plusi & mj) | (minusi & mjn);
    full_adder fa (ppiin, ones, cin, cout, ppip1out);
endmodule

module booth_encoder (qim1, qi, plusi, minusi);
    input qim1, qi;
    output  plusi, minusi;
    assign plusi = qim1 & ~qi;
    assign minusi = ~qim1 & qi;
endmodule

module mult_row (PPI,PPO,pctrl,mctrl,M);
  parameter n=3;
  input [n:0] PPI;
  output [n+1:0] PPO;
  input pctrl,mctrl;
  input [n-1:0] M;
  wire [n-1:0] O;
  wire [n-1:-1] no;
  wire [n:0] carry;
  wire twos;

  generate
    genvar i;
    for (i=0; i<n; i=i+1)
    begin:col
      array_cell ac(PPI[i], M[i], ~M[i], pctrl, mctrl, carry[i], carry[i+1], O[i], PPO[i]);
      assign no[i] = no[i-1] & O[i];
    end
  endgenerate

  assign carry[0] = mctrl;
  //sign extension, is the ones sign (flip only if all ones and we are negating)
  assign twos = O[n-1] ^ (mctrl & no[n-2]);
  full_adder fa(~twos, carry[n], PPI[n], PPO[n+1], PPO[n]);
endmodule

module lab0(M,Q,P,s1,s2,s3);
  parameter n=3;
  input [n-1:0] M,Q;
  output [2*n-1:0] P,s1,s2,s3;
  wire [n-1:0] pctrl, mctrl;
  wire [n:0] beIn;
  wire [n:0] s0;
  wire [n+1:0] sf;
  assign beIn[n:1] = Q[n-1:0];
  assign beIn[0] = 0;

  generate
    genvar i;
    for (i=0; i<n; i=i+1)
    begin:bes
      booth_encoder be(beIn[i],beIn[i+1],pctrl[i],mctrl[i]);
    end
  endgenerate

  mult_row mr1(0,s0,pctrl[0],mctrl[0],M);
  assign s1[n-1:0] = s0[n-1:0];
  assign s1[n] = ~s0[n];
  assign s1[n+1] = s0[n];
  mult_row mr2(s1[n+1:1],s2[n+2:1],pctrl[1],mctrl[1],M);
  mult_row mr3(s2[n+2:2],sf[n+1:0],pctrl[2],mctrl[2],M);
  assign s3[n+2:2] = sf[n:0];
  
  assign P[0]=s1[0];
  assign P[1]=s2[1];
  assign P[2]=s3[2];
  assign P[3]=s3[3];
  assign P[4]=s3[4];
  assign P[5]=s3[5];
endmodule
