// Array Multiplier. Licensed under the terms of the GNU GPL version 2.0 or
// higher.
module full_adder (x, y, cin, cout, s);
    input x, y, cin;
    output s, cout;
    assign cout = x&y | x&cin | y&cin;
    assign s = y ^ cin ^ x;
endmodule

module array_cell (ppiin, mj, mjn, plusi, minusi, cin, cout, ones, ppip1out);
    input ppiin, mj, mjn, plusi, minusi, cin;
    output cout, ppip1out, ones;
    assign ones = (plusi & mj) | (minusi & mjn);
    full_adder fa (ppiin, ones, cin, cout, ppip1out);
endmodule

module booth_encoder (qim1, qi, plusi, minusi);
    input qim1, qi;
    output  plusi, minusi;
    assign plusi = qim1 & ~qi;
    assign minusi = ~qim1 & qi;
endmodule

module mult_row (PPI,PPO,pctrl,mctrl,M);
  parameter n=8;
  input [n:0] PPI;
  output [n+1:0] PPO;
  input pctrl,mctrl;
  input [n-1:0] M;
  wire [n-1:0] O;
  wire [n-1:-1] no;
  wire [n:0] carry;
  wire twos;

  generate
    genvar i;
    for (i=0; i<n; i=i+1)
    begin:col
      array_cell ac(PPI[i], M[i], ~M[i], pctrl, mctrl, carry[i], carry[i+1], O[i], PPO[i]);
      assign no[i] = no[i-1] & O[i];
    end
  endgenerate

  assign carry[0] = mctrl;
  //sign extension, is the ones sign (flip only if all ones and we are negating)
  assign twos = O[n-1] ^ (mctrl & no[n-2]);
  full_adder fa(~twos, carry[n], PPI[n], PPO[n+1], PPO[n]);
endmodule

module lab0(M,Q,P);
  parameter n=8;
  input [n-1:0] M,Q;
  output [2*n-1:0] P;
  wire [n-1:0] pctrl, mctrl;
  wire [n:0] beIn;
  assign beIn[n:1] = Q[n-1:0];
  assign beIn[0] = 0;

  generate
    genvar i;
    for (i=0; i<n; i=i+1)
    begin:bes
      wire [n+1:0] pp;
      booth_encoder be(beIn[i],beIn[i+1],pctrl[i],mctrl[i]);
      if (i==0)
          mult_row mr(0,pp,pctrl[i],mctrl[i],M);
      else if (i==1)
          mult_row mr( {bes[0].pp[n+1],~bes[0].pp[n+1],bes[0].pp[n-1:1]},pp,pctrl[i],mctrl[i],M);
      else 
          mult_row mr(bes[i-1].pp[n+1:1],pp,pctrl[i],mctrl[i],M);
      assign P[i] = pp[0];
    end
  endgenerate
  assign P[2*n-1:n] = bes[n-1].pp[n+1:1];
endmodule
