
// Array Multiplier. Licensed under the terms of the GNU GPL version 2.0 or
// higher.
module full_adder (x, y, cin, cout, sout, s);
    input x, y, cin;
    output s, cout, sout;
    assign cout = x&y | x&cin | y&cin;
    assign sout = y ^ cin;
    assign s = sout ^ x;
endmodule

module array_cell (ppiin, mj, mjn, plusi, minusi, cin, cout, sout, ppip1out);
    input ppiin, mj, mjn, plusi, minusi, cin;
    output cout, ppip1out, sout;
    wire z;
    assign z = (plusi & mj) | (minusi & mjn);
    full_adder fa (ppiin, z, cin, cout, sout, ppip1out);
endmodule

module booth_encoder (qim1, qi, plusi, minusi);
    input qim1, qi;
    output  plusi, minusi;
    assign plusi = qim1 & ~qi;
    assign minusi = ~qim1 & qi;
endmodule

module lab0(M,Q,P,s1,s2,s3);
    input [3-1:0] M, Q;
    wire  [3:0] beIn;
    assign beIn[3:1] = Q[3-1:0];
    assign beIn[0] = 0;

    output [2*3-1:0] P;
    output [2*3-1:0] s1,s2,s3;
    wire [3-1:0]pctrl, mctrl;

        wire top_row0,sin_row0,dc_row0,s_row0,cout_row0;
        booth_encoder be_row0(beIn[0],beIn[0+1],pctrl[0],mctrl[0]); 
          wire ppi_row0_col0, cin_row0_col0,
          cout_row0_col0, sout_row0_col0, ppo_row0_col0;
          array_cell ac_row0_col0(ppi_row0_col0,
                        M[0], ~M[0],
                        pctrl[0],mctrl[0],
                        mctrl[0],cout_row0_col0,
                        sout_row0_col0,ppo_row0_col0);
          wire ppi_row0_col1, cin_row0_col1,
          cout_row0_col1, sout_row0_col1, ppo_row0_col1;
          array_cell ac_row0_col1(ppi_row0_col1,
                        M[1], ~M[1],
                        pctrl[0],mctrl[0],
                        cout_row0_col0,cout_row0_col1,
                        sout_row0_col1,ppo_row0_col1);
          wire ppi_row0_col2, cin_row0_col2,
          cout_row0_col2, sout_row0_col2, ppo_row0_col2;
          array_cell ac_row0_col2(ppi_row0_col2,
                        M[2], ~M[2],
                        pctrl[0],mctrl[0],
                        cout_row0_col1,cout_row0_col2,
                        sout_row0_col2,ppo_row0_col2);

        wire top_row1,sin_row1,dc_row1,s_row1,cout_row1;
        booth_encoder be_row1(beIn[1],beIn[1+1],pctrl[1],mctrl[1]); 
          wire ppi_row1_col0, cin_row1_col0,
          cout_row1_col0, sout_row1_col0, ppo_row1_col0;
          array_cell ac_row1_col0(ppi_row1_col0,
                        M[0], ~M[0],
                        pctrl[1],mctrl[1],
                        mctrl[1],cout_row1_col0,
                        sout_row1_col0,ppo_row1_col0);
          wire ppi_row1_col1, cin_row1_col1,
          cout_row1_col1, sout_row1_col1, ppo_row1_col1;
          array_cell ac_row1_col1(ppi_row1_col1,
                        M[1], ~M[1],
                        pctrl[1],mctrl[1],
                        cout_row1_col0,cout_row1_col1,
                        sout_row1_col1,ppo_row1_col1);
          wire ppi_row1_col2, cin_row1_col2,
          cout_row1_col2, sout_row1_col2, ppo_row1_col2;
          array_cell ac_row1_col2(ppi_row1_col2,
                        M[2], ~M[2],
                        pctrl[1],mctrl[1],
                        cout_row1_col1,cout_row1_col2,
                        sout_row1_col2,ppo_row1_col2);

        wire top_row2,sin_row2,dc_row2,s_row2,cout_row2;
        booth_encoder be_row2(beIn[2],beIn[2+1],pctrl[2],mctrl[2]); 
          wire ppi_row2_col0, cin_row2_col0,
          cout_row2_col0, sout_row2_col0, ppo_row2_col0;
          array_cell ac_row2_col0(ppi_row2_col0,
                        M[0], ~M[0],
                        pctrl[2],mctrl[2],
                        mctrl[2],cout_row2_col0,
                        sout_row2_col0,ppo_row2_col0);
          wire ppi_row2_col1, cin_row2_col1,
          cout_row2_col1, sout_row2_col1, ppo_row2_col1;
          array_cell ac_row2_col1(ppi_row2_col1,
                        M[1], ~M[1],
                        pctrl[2],mctrl[2],
                        cout_row2_col0,cout_row2_col1,
                        sout_row2_col1,ppo_row2_col1);
          wire ppi_row2_col2, cin_row2_col2,
          cout_row2_col2, sout_row2_col2, ppo_row2_col2;
          array_cell ac_row2_col2(ppi_row2_col2,
                        M[2], ~M[2],
                        pctrl[2],mctrl[2],
                        cout_row2_col1,cout_row2_col2,
                        sout_row2_col2,ppo_row2_col2);

        assign ppi_row0_col0 = 0;
        assign ppi_row0_col1 = 0;
        assign ppi_row0_col2 = 0;
        assign ppi_row1_col0 = ppo_row0_col1; 
        assign ppi_row1_col1 = ppo_row0_col2; 
        assign ppi_row1_col2 = sout_row0_col2;
        assign ppi_row2_col0 = ppo_row1_col1; 
        assign ppi_row2_col1 = ppo_row1_col2; 
      assign top_row2 = ~sout_row1_col2;
      assign sin_row2 = ~sout_row2_col2;

      assign s1[0] = sout_row0_col0;
      assign s1[1] = sout_row0_col1;
      assign s1[2] = sout_row0_col2;
      assign s1[3] = sout_row0_col2;
      assign s1[4] = ~sout_row0_col2;
      assign s1[5] = 0;

      assign s2[0] = 0;
      assign s2[1] = sout_row1_col0;
      assign s2[2] = sout_row1_col1;
      assign s2[3] = sout_row1_col2;
      assign s2[4] = ~sout_row1_col2;
      assign s2[5] = 0;

      assign s3[0] = 0;
      assign s3[1] = 0;
      assign s3[2] = sout_row2_col0;
      assign s3[3] = sout_row2_col1;
      assign s3[4] = sout_row2_col2;
      assign s3[5] = ~sout_row2_col2;

      assign P[0] = ppo_row0_col0;
      assign P[1] = ppo_row1_col0;
      assign P[2] = ppo_row2_col0;
      assign P[3] = ppo_row2_col1;
      assign P[4] = ppo_row2_col2;
      assign P[5] = s_row2;
endmodule
