// Full Adder
module full_adder (x, y, cin, cout, sout, s);
    input x, y, cin;
    output s, cout, sout;
    assign cout = x&y | x&cin | y&cin;
    assign sout = y ^ cin;
    assign s = sout ^ x;
endmodule

module array_cell (ppiin, mj, mjn, plusi, minusi, cin, cout, sout, ppip1out);
    input ppiin, mj, mjn, plusi, minusi, cin;
    output cout, ppip1out, sout;
    wire z;
    assign z = (plusi & mj) | (minusi & mjn);
    full_adder fa (ppiin, z, cin, cout, sout, ppip1out);
endmodule

module booth_encoder (qim1, qi, plusi, minusi);
    input qim1, qi;
    output  plusi, minusi;
    assign plusi = qim1 & ~qi;
    assign minusi = ~qim1 & qi;
endmodule

module lab0(M,Q,P,pctrl,mctrl);
    parameter n=2;
    input [n-1:0] M, Q;
    wire  [n:0] beIn;
    assign beIn[n:1] = Q[n-1:0];
    assign beIn[0] = 0;

    output [2*n-1:0] P;
    output [n-1:0]pctrl, mctrl;

    genvar r, c;

    generate
      for (r=0; r<n; r=r+1)
      begin:row
        wire top,sin,dc,s,cout;
        booth_encoder be(beIn[r],beIn[r+1],pctrl[r],mctrl[r]);
        for (c=0; c<n; c=c+1)
        begin:col
          wire ppi, cin, cout, sout, ppo;
          array_cell ac(ppi, M[c], ~M[c], pctrl[r],mctrl[r],cin,cout,sout,ppo);
        end //col
        full_adder fa(top,sin,0, cout,dc,s);
      end //row

      for (r=0;r<n;r=r+1)
      begin
        for (c=0;c<n;c=c+1)
        begin
          if (r == 0)
            assign row[r].col[c].ppi = 0;
          else if (c != (n-1))
            assign row[r].col[c].ppi = row[r-1].col[c+1].ppo;
          else if (r == 1)
            assign row[r].col[c].ppi = row[r-1].col[c+1].sout;
        end
        if (r == 2)
        begin
          assign row[r].top = ~row[r-1].col[n-1].sout;
          assign row[r].sin = ~row[r].col[n-1].sout;
        end
        else if (r > 2)
        begin
          assign row[r].sin = ~row[r].col[n-1].sout;
          assign row[r].top = row[r-1].cout;
          assign row[r].col[n-1].ppi = row[r-1].s;
        end
      end
    endgenerate
endmodule
