// Array Multiplier. Licensed under the terms of the GNU GPL version 2.0 or
// higher.
module full_adder (x, y, cin, cout, s);
    input x, y, cin;
    output s, cout;
    assign cout = x&y | x&cin | y&cin;
    assign s = y ^ cin ^ x;
endmodule

module mult_row (X,Y,CIN,S,COUT);
  parameter n=8;
  input [n-1:0] PPI,M,CIN;
  output [n-1:0] PPO,COUT;
  generate
    genvar i;
    for (i=0; i<n; i=i+1)
    begin:col
      full_adder fa(X[i],Y[i], CIN[i],COUT[i],S[i]);
    end
  endgenerate
endmodule

module lab0-carry(M,Q,P);
  parameter n=8;
  input [n-1:0] M,Q;
  output [2*n-1:0] P;
  wire [n-1:0] co1,co2,co3,co4,co5,co6,co7,co8;
  wire [n-1:0] s1,s2,s3,s4,s5,s6,s7,s8;

  mult_row({0,         M[7]&Q[0], M[6]&Q[0], M[5]&Q[0], M[4]&Q[0], M[3]&Q[0], M[2]&Q[0], M[1]&Q[0]},
           {M[7]&Q[1], M[6]&Q[1], M[5]&Q[1], M[4]&Q[1], M[3]&Q[1], M[2]&Q[1], M[1]&Q[1], M[0]&Q[1]},
           {M[6]&Q[2], M[5]&Q[2], M[4]&Q[2], M[3]&Q[2], M[2]&Q[2], M[1]&Q[2], M[0]&Q[2], 0        },
           s1, co1);
  mult_row({M[7]&Q[2], s1[7:1]},
           {M[6]&Q[3], M[5]&Q[3], M[4]&Q[3], M[3]&Q[3], M[2]&Q[3], M[1]&Q[3], M[0]&Q[3], 0        },
           co1,
           s2, co2);
  mult_row({M[7]&Q[3], s2[7:1]},
           {M[6]&Q[4], M[5]&Q[4], M[4]&Q[4], M[3]&Q[4], M[2]&Q[4], M[1]&Q[4], M[0]&Q[4], 0        },
           co2,
           s3, co3);
  mult_row({M[7]&Q[4], s3[7:1]},
           {M[6]&Q[5], M[5]&Q[5], M[4]&Q[5], M[3]&Q[5], M[2]&Q[5], M[1]&Q[5], M[0]&Q[5], 0        },
           co3,
           s4, co4);
  mult_row({M[7]&Q[5], s4[7:1]},
           {M[6]&Q[6], M[5]&Q[6], M[4]&Q[6], M[3]&Q[6], M[2]&Q[6], M[1]&Q[6], M[0]&Q[6], 0        },
           co4,
           s5, co5);
  mult_row({M[7]&Q[6], s5[7:1]},
           {M[6]&Q[7], M[5]&Q[7], M[4]&Q[7], M[3]&Q[7], M[2]&Q[7], M[1]&Q[7], M[0]&Q[7], 0        },
           co4,
           s5, co5);
  mult_row({M[7]&Q[7], s6[7:1]},
           co5,
           {co6[6:0],0},
           P[14:6], {P[15], co6[6:0]});

  assign P[0] = M[0]&Q[0];
  assign P[1] = s1[0];
  assign P[2] = s2[0];
  assign P[3] = s3[0];
  assign P[4] = s4[0];
  assign P[5] = s5[0];

endmodule
