
// Array Multiplier. Licensed under the terms of the GNU GPL version 2.0 or
// higher.
module full_adder (x, y, cin, cout, s);
    input x, y, cin;
    output s, cout;
    assign cout = x&y | x&cin | y&cin;
    assign s = y ^ cin ^ x;
endmodule

module array_cell (ppiin, mj, mjn, plusi, minusi, cin, cout, ones, ppip1out);
    input ppiin, mj, mjn, plusi, minusi, cin;
    output cout, ppip1out, ones;
    assign ones = (plusi & mj) | (minusi & mjn);
    full_adder fa (ppiin, ones, cin, cout, ppip1out);
endmodule

module booth_encoder (qim1, qi, plusi, minusi);
    input qim1, qi;
    output  plusi, minusi;
    assign plusi = qim1 & ~qi;
    assign minusi = ~qim1 & qi;
endmodule

module mult_row (PPI,PPO,pctrl,mctrl,M);
  parameter n=8;
  input [n:0] PPI;
  output [n+1:0] PPO;
  input pctrl,mctrl;
  input [n-1:0] M;
  wire [n-1:0] O;
  wire [n-1:-1] no;
  wire [n:0] carry;
  wire twos;

  generate
    genvar i;
    for (i=0; i<n; i=i+1)
    begin:col
      array_cell ac(PPI[i], M[i], ~M[i], pctrl, mctrl, carry[i], carry[i+1], O[i], PPO[i]);
      assign no[i] = no[i-1] & O[i];
    end
  endgenerate

  assign carry[0] = mctrl;
  //sign extension, is the ones sign (flip only if all ones and we are negating)
  assign twos = O[n-1] ^ (mctrl & no[n-2]);
  full_adder fa(~twos, carry[n], PPI[n], PPO[n+1], PPO[n]);
endmodule

module lab0(M,Q,P);
  parameter n=8;
  input [n-1:0] M,Q;
  output [2*n-1:0] P;
  wire [n-1:0] pctrl, mctrl;
  wire [n:0] beIn;
  assign beIn[n:1] = Q[n-1:0];
  assign beIn[0] = 0;

      wire [n+1:0] pp0;
      booth_encoder be(beIn[0],beIn[1],pctrl[0],mctrl[0]);
     mult_row mr(0,pp0,pctrl[0],mctrl[0],M);

      wire [n+1:0] pp1;
      booth_encoder be(beIn[1],beIn[2],pctrl[1],mctrl[1]);
      mult_row mr({pp0[n+1],~pp0[n+1],pp0[n-1:1]},pp1,pctrl[1],mctrl[1],M);

      wire [n+1:0] pp2;
      booth_encoder be(beIn[2],beIn[3],pctrl[2],mctrl[2]);
      mult_row mr(pp1[n+1:1],pp2,pctrl[2],mctrl[2],M);
      assign P[2] = pp2[0];

      wire [n+1:0] pp3;
      booth_encoder be(beIn[3],beIn[4],pctrl[3],mctrl[3]);
      mult_row mr(pp2[n+1:1],pp3,pctrl[3],mctrl[3],M);
      assign P[3] = pp3[0];

      wire [n+1:0] pp4;
      booth_encoder be(beIn[4],beIn[5],pctrl[4],mctrl[4]);
      mult_row mr(pp3[n+1:1],pp4,pctrl[4],mctrl[4],M);
      assign P[4] = pp4[0];

      wire [n+1:0] pp5;
      booth_encoder be(beIn[5],beIn[6],pctrl[5],mctrl[5]);
      mult_row mr(pp4[n+1:1],pp5,pctrl[5],mctrl[5],M);
      assign P[5] = pp5[0];

      wire [n+1:0] pp6;
      booth_encoder be(beIn[6],beIn[7],pctrl[6],mctrl[6]);
      mult_row mr(pp5[n+1:1],pp6,pctrl[6],mctrl[6],M);
      assign P[6] = pp6[0];

      wire [n+1:0] pp7;
      booth_encoder be(beIn[7],beIn[8],pctrl[7],mctrl[7]);
      mult_row mr(pp6[n+1:1],pp7,pctrl[7],mctrl[7],M);
      assign P[7] = pp7[0];
   assign P[2*n-1:n] = pp7[n+1:1];
endmodule
