#!/usr/bin/python
"""
Assembler for our pet processor.
"""

import sys,re

if len(sys.argv) <> 2:
    print "usage: asm.py <sourcefile>"
    sys.exit(1);

stdout = sys.stdout
sys.stdout = file(sys.argv[1]+'.mif', 'w')
indata = file(sys.argv[1]).readlines()

registers = {'r0' : '00',
             'r1' : '01',
             'r2' : '10',
             'r3' : '11'}

instructions = {'movi'  : '0000',
                'move'  : '0001',
                'load'  : '0010',
                'add'   : '0011',
                'sub'   : '0100',
                'halt'  : '0101',
                'store' : '0110'}

lines = [r'(movi)\s+(r[0-3])\s+([hbd]?[0-9a-fA-F]+)',
         r'(move)\s+(r[0-3])\s+(r[0-3])',
         r'(load)\s+(r[0-3])',
         r'(store)\s+(r[0-3])',
         r'(add)\s+(r[0-3])\s+(r[0-3])',
         r'(sub)\s+(r[0-3])\s+(r[0-3])',
         r'(halt)']

magic = [re.compile(x) for x in lines]

print """DEPTH = 32;
WIDTH = 16;
ADDRESS_RADIX = HEX;
DATA_RADIX = BINARY;

CONTENT
BEGIN
	[10..1F] : 0000;"""

def tobin(n,b):
    r=''
    for t in range(b-1,-1,-1):
        if 2**t <= n:
            r += '1'
            n -= 2**t
        else:
            r += '0'
    return r

def parsedata(data):
    if data[0] == 'h':
        return tobin(int(data[1:],16),8)
    elif data[0] == 'd':
        return tobin(int(data[1:]),8)
    elif data[0] == 'b':
        if len(data[1:]) == 8 and re.match('[01]+', data[1:]):
            return data[1:]
    return ''

pc = 0;
i = 1
ln=1;
for i in indata:

    match = [x.match(i) for x in magic if x.match(i)]

    if len(match) == 1:

        type = match[0].group(1)
        ins = instructions[type]
        data = '0'*8
        rx = '00'
        ry = '00'

        if type == 'movi':
            type, r, data = match[0].groups()
            data = parsedata(data)
            rx = registers[r]
        elif type == 'load' or type == 'store':
            type, r = match[0].groups()
            rx = registers[r]
        elif type in ['add','sub','move']:
            type, rx, ry = match[0].groups()
            rx = registers[rx]
            ry = registers[ry]

        print "     %X : %s%s%s%s; %% %s %%" % (pc, ins, rx, ry, data, i[:-1])
        pc += 1

    else:
        print >> sys.stderr, "Syntax error line %i:"%ln, i,
    ln += 1

print "END;"
