module control( ir, clk, m68k_master, cur, next,
                pc_r0, incr, add_sub, read, write,
                pci, r0i, r1i, r2i, r3i, iri, zzi,
                pco, iro, zzo, r0o, r1o, r2o, r3o, sramo,
                splice);

  input[15:0] ir;
  input clk, m68k_master;

  output pc_r0, incr, add_sub, read, write,
         pci, r0i, r1i, r2i, r3i, iri, zzi,
         pco, iro, zzo, r0o, r1o, r2o, r3o, sramo, splice;
  output[7:0] cur,next;

  reg[7:0] cur,next;
  reg[20:0] out;
  assign { pc_r0, incr, add_sub, read, write,
           pci, r0i, r1i, r2i, r3i, iri, zzi,
           pco, iro, zzo, r0o, r1o, r2o, r3o, sramo, splice} = out;

  parameter PC_R0   = 21'b100000000000000000000,
            INCR    = 21'b010000000000000000000,
            ADD_SUB = 21'b001000000000000000000,
            READ    = 21'b000100000000000000000,
            WRITE   = 21'b000010000000000000000,
            PCI     = 21'b000001000000000000000,
            R0I     = 21'b000000100000000000000,
            R1I     = 21'b000000010000000000000,
            R2I     = 21'b000000001000000000000,
            R3I     = 21'b000000000100000000000,
            IRI     = 21'b000000000010000000000,
            ZZI     = 21'b000000000001000000000,
            PCO     = 21'b000000000000100000000,
            IRO     = 21'b000000000000010000000,
            ZZO     = 21'b000000000000001000000,
            R0O     = 21'b000000000000000100000,
            R1O     = 21'b000000000000000010000,
            R2O     = 21'b000000000000000001000,
            R3O     = 21'b000000000000000000100,
            SRAMO   = 21'b000000000000000000010,
            SPLICE  = 21'b000000000000000000001;
  
  parameter
    movi=4'h0, move=4'h1, load=4'h2, store=4'h3, add=4'h4, sub=4'h5, halt=4'h6,
    ifetch_1=8'h00, ifetch_2=8'h01, ifetch_3=8'h02, ifetch_4=8'h03,
    movi_1=8'h10, movi_2=8'h11, move_1=8'h20, load_1=8'h30, store_1=8'h40,
    add_1=8'h50, sub_1=8'h60, load_2=8'h31, store_2=8'h42, add_2=8'h52,
    add_3=8'h53, sub_2=8'h62, sub_3=8'h63, terminate=8'hFF, m68kwait=8'hFE;

  always @(cur)
    case(cur)
      m68kwait:  next <= ifetch_1;
      ifetch_1:  next <= ifetch_2;
      ifetch_2:  next <= ifetch_3;
      ifetch_3:  next <= ifetch_4;
      ifetch_4:  case (ir[15:12])
                   movi:  next <= movi_1; //no more states necessary
                   move:  next <= move_1; //no more states necessary
                   load:  next <= load_1; 
                   store: next <= store_1;
                   add:   next <= add_1;
                   sub:   next <= sub_1;
                   halt:  next <= terminate; //no more states necessary
                 endcase
      movi_1:    next <= movi_2;
      load_1:    next <= load_2;
      store_1:   next <= store_2;
      add_1:     next <= add_2;
      add_2:     next <= add_3;
      sub_1:     next <= sub_2;
      sub_2:     next <= sub_3;
      terminate: next <= terminate;
      default:   next <= ifetch_1;
    endcase
           
  // IR breakdown:
  // | 15:12 opcode | 11:10 Rx | 9:8 | Ry | 7:0 IMM data |
  //
  // name        opcode      action
  // ----        ------      ------
  // move         0000       rx <- data
  // move         0001       rx <- ry
  // load         0010       rx <- (r0)
  // store        0011       (r0) <- ry
  // add          0100       rx <- rx + ry
  // sub          0101       rx <- rx - ry
  // halt         0110       stop executing; fetch no more instructions

  always @(posedge clk)
    if (~m68k_master)
    begin
      cur = next; //maybe should be blocking?
      case (cur)
        ifetch_1: out <= PC_R0 | READ;
        ifetch_2: out <= SRAMO | IRI;
        ifetch_3: out <= PCO   | INCR;
        ifetch_4: out <= ZZO   | PCI;

        movi_1: out <= IRO | SPLICE | ZZI;
        movi_2: case (ir[11:10])
                  2'b00: out <= ZZO | R0I;
                  2'b01: out <= ZZO | R1I;
                  2'b10: out <= ZZO | R2I;
                  2'b11: out <= ZZO | R3I;
                endcase
        move_1: case (ir[11:10])
                  2'b00: case (ir[9:8])
                          2'b00: out <= R0O | R0I;
                          2'b01: out <= R1O | R0I;
                          2'b10: out <= R2O | R0I;
                          2'b11: out <= R3O | R0I;
                         endcase
                  2'b01: case (ir[9:8])
                          2'b00: out <= R0O | R1I;
                          2'b01: out <= R1O | R1I;
                          2'b10: out <= R2O | R1I;
                          2'b11: out <= R3O | R1I;
                         endcase
                  2'b10: case (ir[9:8])
                          2'b00: out <= R0O | R2I;
                          2'b01: out <= R1O | R2I;
                          2'b10: out <= R2O | R2I;
                          2'b11: out <= R3O | R2I;
                         endcase
                  2'b11: case (ir[9:8])
                          2'b00: out <= R0O | R3I;
                          2'b01: out <= R1O | R3I;
                          2'b10: out <= R2O | R3I;
                          2'b11: out <= R3O | R3I;
                         endcase
                endcase
        load_1: case (ir[11:10])
                  2'b00: out <= READ | SRAMO | R0I;
                  2'b01: out <= READ | SRAMO |R1I;
                  2'b10: out <= READ | SRAMO |R2I;
                  2'b11: out <= READ | SRAMO |R3I;
                endcase
        load_2: ;
        store_1: case (ir[9:8])
                  2'b00: out <= R0O | WRITE;
                  2'b01: out <= R1O | WRITE;
                  2'b10: out <= R2O | WRITE;
                  2'b11: out <= R3O | WRITE;
                endcase
        store_2: ;
        add_1:  case (ir[11:10])
                  2'b00: out <= R0O;
                  2'b01: out <= R1O;
                  2'b10: out <= R2O;
                  2'b11: out <= R3O;
                endcase
        add_2:  case (ir[9:8])
                  2'b00: out <= R0O | ADD_SUB | ZZI;
                  2'b01: out <= R1O | ADD_SUB | ZZI;
                  2'b10: out <= R2O | ADD_SUB | ZZI;
                  2'b11: out <= R3O | ADD_SUB | ZZI;
                endcase
        add_3:  case (ir[11:10])
                  2'b00: out <= R0I | ZZO;
                  2'b01: out <= R1I | ZZO;
                  2'b10: out <= R2I | ZZO;
                  2'b11: out <= R3I | ZZO;
                endcase
        sub_1:  case (ir[11:10])
                  2'b00: out <= R0O;
                  2'b01: out <= R1O;
                  2'b10: out <= R2O;
                  2'b11: out <= R3O;
                endcase
        sub_2:  case (ir[9:8])
                  2'b00: out <= R0O | ZZI;
                  2'b01: out <= R1O | ZZI;
                  2'b10: out <= R2O | ZZI;
                  2'b11: out <= R3O | ZZI;
                endcase
        sub_3:  case (ir[11:10])
                  2'b00: out <= R0I | ZZO;
                  2'b01: out <= R1I | ZZO;
                  2'b10: out <= R2I | ZZO;
                  2'b11: out <= R3I | ZZO;
                endcase
        terminate: out <= 0;
        default: out <= 0;
      endcase
    end
    else
      cur <= m68kwait;

endmodule
