module sram (m68k_master, srama, sramdl, sramdh, sram1en, sram2en, sram1oe, sram2oe,
             sram1we, sram2we, sram1ud, sram2ud, sram1ld, sram2ld, address, as, rw, uds, lds,
             dtack, data_68k, proc_addr, proc_read, proc_write, from_proc, to_proc);

  input m68k_master;
  output [17:0] srama;
  inout [15:0] sramdl, sramdh;

  output sram1en, sram2en;
  output sram1oe, sram2oe;
  output sram1we, sram2we;
  output sram1ud, sram2ud;
  output sram1ld, sram2ld;

  input [23:1] address;
  input as, rw, uds, lds;
  inout dtack;
  inout [15:0] data_68k;

  input [8:0] proc_addr;
  input proc_read, proc_write;
  input [15:0] from_proc;
  output [15:0] to_proc;
      
  wire ack, gnd;
  wire sram1_sel, sram2_sel;
  wire sram1_read, sram2_read;
  wire sram1_write, sram2_write;
  reg [15:0] data_reg, sramdl_reg, sramdh_reg;
  reg [15:0] to_proc;

  assign gnd = 1'b0;
  assign dtack = ack ? gnd : 'bz;
    
  assign srama[17:0] = m68k_master ? address[18:1] : { 9'b0, proc_addr[8:0] };

  // Replace the "..." with the appropriate address bits
  // Both signals are active high

  // default behaviour is sramse1 when m68kmaster is low
  // se1 = A[0..7]xxxx, se2 = A[8..F]xxxx
  assign sram1_sel = ( (5'b10100 == address[23:19]) & (~ as) & m68k_master ) | (~ m68k_master);

  assign sram2_sel = (5'b10101 == address[23:19]) & (~ as) & m68k_master;

  // Complete the expressions for sram1_read and sram1_write
  // Hint: sram1_read and sram1_write need to include the signals
  //       proc_read and proc_write respectively

  assign sram1_read = sram1_sel & (proc_read | rw);
  assign sram2_read = sram2_sel & rw;

  assign sram1_write = sram1_sel & (proc_write | ~rw);
  assign sram2_write = sram2_sel & (~ rw);
   
  assign sram1ud = m68k_master ? uds : 1'b0;
  assign sram2ud = m68k_master ? uds : 1'b1;
  assign sram1ld = m68k_master ? lds : 1'b0;
  assign sram2ld = m68k_master ? lds : 1'b1;
   
  assign ack = ( sram1_read | sram2_read | sram1_write | sram2_write ) & m68k_master;

  always @( sram1_read or sram2_read or sram1_write or sram2_write or sramdl or sramdh )
  begin
    data_reg    = 16'bz;
    to_proc     = 16'b0;
    sramdl_reg  = 16'bz;
    sramdh_reg  = 16'bz;

    // Complete the assignments to data_reg, to_proc, sramdl_reg, sramdh_reg
        // Note: data_reg, sramdl_reg and sramdh_reg are the registered version of data, sramdl and sramdh, respectively

    if( sram1_read == 1'b1 )
    begin 
      if( m68k_master ==1'b1 )  
        data_reg = sramdl;
      else
        to_proc = sramdl;
    end
    else if( sram2_read == 1'b1 )
    begin 
      if( m68k_master == 1'b1 ) 
        data_reg = sramdh;
      else
        to_proc = 4'hF;
    end
    else if( sram1_write == 1'b1 )
    begin 
      if( m68k_master == 1'b1 ) 
        sramdl_reg = data_68k;
      else 
        sramdl_reg = from_proc;
      end
    else if( sram2_write == 1'b1 )
    begin 
      if( m68k_master ==1'b1 ) 
        sramdh_reg = data_68k;
/*      else
        sramdh_reg = ;*/
    end     
  end   // always block
   
  assign data_68k = data_reg;
  assign sramdl     = sramdl_reg;
  assign sramdh     = sramdh_reg;

  assign sram1en = ~ sram1_sel,
         sram2en = ~ sram2_sel,
         sram1oe = ~ sram1_read,
         sram2oe = ~ sram2_read,
         sram1we = ~ sram1_write,
         sram2we = ~ sram2_write;

endmodule
